.. Do not edit this file.  This file is auto-generated for PyPI by setup.py
.. using pandoc, so edits should go in the source files rather than here.

Pystache
========

|CI| |Conda| |Coverage| |Security check - Bandit| |Release|

|pre-commit| |Maintainability|

|Latest release| |License| |Python|

This updated fork of Pystache is currently tested on Python 3.6+ and in
Conda, on Linux, Macos, and Windows (Python 2.7 support has been
removed).

.. image:: gh/images/logo_phillips_small.png

`Pystache <http://PennyDreadfulMTG.github.com/pystache>`__ is a Python
implementation of `Mustache <http://mustache.github.com/>`__. Mustache
is a framework-agnostic, logic-free templating system inspired by
`ctemplate <https://code.google.com/p/google-ctemplate/>`__ and et by
Ivan Formichev. Like ctemplate, Mustache "emphasizes separating logic
from presentation: it is impossible to embed application logic in this
template language."

The `mustache(5) <https://mustache.github.com/mustache.5.html>`__ man
page provides a good introduction to Mustache's syntax. For a more
complete (and more current) description of Mustache's behavior, see the
official `Mustache spec <https://github.com/mustache/spec>`__.

Pystache is `semantically versioned <http://semver.org>`__ and older
versions can still be found on
`PyPI <http://pypi.python.org/pypi/pystache>`__. This version of
Pystache now passes all tests in `version
1.1.3 <https://github.com/mustache/spec/tree/v1.1.3>`__ of the spec.

Making Changes & Contributing
-----------------------------

This repo is now pre-commit enabled for various linting and format
checks (and in many cases, automatic fixes). The checks run on
commit/push and will fail the commit (if not clean) with some checks
performing simple file corrections. Simply review the changes and adjust
(if needed) then ``git add`` the files and continue.

If other checks fail on commit, the failure display should explain the
error types and line numbers. Note you must fix any fatal errors for the
commit to succeed; some errors should be fixed automatically (use
``git status`` and ``git diff`` to review any changes).

Note ``pylint`` is the primary check that requires your own input, as
well as a decision as to the appropriate fix action. You must fix any
``pylint`` warnings (relative to the baseline config score) for the
commit to succeed.

See the pre-commit docs under ``docs/dev/`` for more information.

-  `pre-commit-config <docs/dev/pre-commit-config.rst>`__
-  `pre-commit-usage <docs/dev/pre-commit-usage.rst>`__

You will need to install pre-commit before contributing any changes;
installing it using your system's package manager is recommended,
otherwise install with pip into your local user's virtual environment
using something like:

::

   $ sudo emerge pre-commit  --or--
   $ pip install pre-commit

then install it into the repo you just cloned:

::

   $ git clone https://github.com/PennyDreadfulMTG/pystache
   $ cd pystache
   $ pre-commit install
   $ pre-commit install-hooks

It's usually a good idea to update the hooks to the latest version:

::

   pre-commit autoupdate

Requirements
------------

Pystache is tested with--

-  Python 3.6
-  Python 3.7
-  Python 3.8
-  Python 3.9
-  Conda (py36-py39)

`Distribute <http://packages.python.org/distribute/>`__ (the setuptools
fork) is no longer required over
`setuptools <http://pypi.python.org/pypi/setuptools>`__, as the current
packaging is now PEP517-compliant.

JSON support is needed only for the command-line interface and to run
the spec tests; PyYAML can still be used (see the Develop section).

Official support for Python 2 will end with Pystache version 0.6.0.

Install It
----------

::

   pip install -U pystache -f https://github.com/PennyDreadfulMTG/pystache/releases/

And test it:

::

   pystache-test

To install and test from source (e.g. from GitHub), see the Develop
section.

Use It
------

::

   >>> import pystache
   >>> print(pystache.render('Hi {{person}}!', {'person': 'Mom'}))
   Hi Mom!

You can also create dedicated view classes to hold your view logic.

Here's your view class (in ../pystache/tests/examples/readme.py):

::

   class SayHello(object):
       def to(self):
           return "Pizza"

Instantiating like so:

::

   >>> from pystache.tests.examples.readme import SayHello
   >>> hello = SayHello()

Then your template, say_hello.mustache (by default in the same directory
as your class definition):

::

   Hello, {{to}}!

Pull it together:

::

   >>> renderer = pystache.Renderer()
   >>> print(renderer.render(hello))
   Hello, Pizza!

For greater control over rendering (e.g. to specify a custom template
directory), use the ``Renderer`` class like above. One can pass
attributes to the Renderer class constructor or set them on a Renderer
instance. To customize template loading on a per-view basis, subclass
``TemplateSpec``. See the docstrings of the
`Renderer <https://github.com/PennyDreadfulMTG/pystache/blob/master/pystache/renderer.py>`__
class and
`TemplateSpec <https://github.com/PennyDreadfulMTG/pystache/blob/master/pystache/template_spec.py>`__
class for more information.

You can also pre-parse a template:

::

   >>> parsed = pystache.parse(u"Hey {{#who}}{{.}}!{{/who}}")
   >>> print(parsed)
   ['Hey ', _SectionNode(key='who', index_begin=12, index_end=18, parsed=[_EscapeNode(key='.'), '!'])]

And then:

::

   >>> print(renderer.render(parsed, {'who': 'Pops'}))
   Hey Pops!
   >>> print(renderer.render(parsed, {'who': 'you'}))
   Hey you!

Unicode
-------

This section describes how Pystache handles unicode, strings, and
encodings.

Internally, Pystache uses `only unicode
strings <https://docs.python.org/howto/unicode.html#tips-for-writing-unicode-aware-programs>`__.
For input, Pystache accepts both unicode strings and byte strings. For
output, Pystache's template rendering methods return only unicode.

Pystache's ``Renderer`` class supports a number of attributes to control
how Pystache converts byte strings to unicode on input. These include
the ``file_encoding``, ``string_encoding``, and ``decode_errors``
attributes.

The ``file_encoding`` attribute is the encoding the renderer uses to
convert to unicode any files read from the file system. Similarly,
``string_encoding`` is the encoding the renderer uses to convert any
other byte strings encountered during the rendering process into unicode
(e.g. context values that are encoded byte strings).

The ``decode_errors`` attribute is what the renderer passes as the
``errors`` argument to ``str()``. The valid values for this argument are
``strict``, ``ignore``, and ``replace``.

Each of these attributes can be set via the ``Renderer`` class's
constructor using a keyword argument of the same name. See the Renderer
class's docstrings for further details. In addition, the
``file_encoding`` attribute can be controlled on a per-view basis by
subclassing the ``TemplateSpec`` class. When not specified explicitly,
these attributes default to values set in Pystache's ``defaults``
module.

Develop
-------

To test from a source distribution (without installing)--

::

   python test_pystache.py

To test Pystache with multiple versions of Python (with a single
command!) and different platforms, you can use
`tox <http://pypi.python.org/pypi/tox>`__:

::

   pip install tox
   tox -e setup

To run tests on multiple versions with coverage, run:

::

   tox -e py38-linux,py39-linux  # for example

(substitute your platform above, eg, macos or windows)

The source distribution tests also include doctests and tests from the
Mustache spec. To include tests from the Mustache spec in your test
runs:

::

   git submodule update --init

The test harness parses the spec's (more human-readable) yaml files if
`PyYAML <https://pypi.python.org/pypi/PyYAML>`__ is present. Otherwise,
it parses the json files. To install PyYAML--

::

   pip install pyyaml

Once the submodule is available, you can run the full test set with:

::

   tox -e setup . ext/spec/specs

To run a subset of the tests, you can use
`nose <https://somethingaboutorange.com/mrl/projects/nose/0.11.1/testing.html>`__:

::

   pip install nose
   nosetests --tests pystache/tests/test_context.py:GetValueTests.test_dictionary__key_present

Mailing List (old)
------------------

There is(was) a `mailing
list <http://librelist.com/browser/pystache/>`__. Note that there is a
bit of a delay between posting a message and seeing it appear in the
mailing list archive.

Credits
-------

::

   >>> import pystache
   >>> context = { 'author': 'Chris Wanstrath', 'maintainer': 'Chris Jerdonek','refurbisher': 'Steve Arnold', 'new_maintainer': 'Thomas David Baker' }
   >>> print(pystache.render("Author: {{author}}\nMaintainer: {{maintainer}}\nRefurbisher: {{refurbisher}}\nNew maintainer: {{new_maintainer}}", context))
   Author: Chris Wanstrath
   Maintainer: Chris Jerdonek
   Refurbisher: Steve Arnold
   New maintainer: Thomas David Baker

Pystache logo by `David Phillips <https://davidphillips.us/>`__ is
licensed under a `Creative Commons Attribution-ShareAlike 3.0 Unported
License <https://creativecommons.org/licenses/by-sa/3.0/deed.en_US>`__.
|image1|

History
=======

**Note:** Official support for Python 2.7 will end with Pystache version
0.6.0.

0.6.0 (2021-03-04)
------------------

-  Bump spec versions to latest => v1.1.3
-  Modernize python and CI tools, update docs/doctests
-  Update unicode conversion test for py3-only
-  Add pep8speaks cfg, cleanup warnings
-  Remove superfluous setup test/unused imports
-  Add conda recipe/CI build

.. _section-1:

0.5.6 (2021-02-28)
------------------

-  Use correct wheel name in release workflow, limit wheels
-  Add install check/test of downloaded wheel
-  Update/add ci workflows and tox cfg, bump to next dev0 version

.. _section-2:

0.5.5 (2020-12-16)
------------------

-  fix document processing, update pandoc args and history
-  add release.yml to CI, test env settings
-  fix bogus commit message, update versions and tox cf
-  add post-test steps for building pkgs with/without doc updates
-  add CI build check, fix MANIFEST.in pruning

.. _section-3:

0.5.4-2 (2020-11-09)
--------------------

-  Merge pull request #1 from sarnold/rebase-up
-  Bugfix: test_specloader.py: fix test_find__with_directory on other
   OSs
-  Bugfix: pystache/loader.py: remove stray windows line-endings
-  fix crufty (and insecure) http urls
-  Bugfix: modernize python versions (keep py27) and fix spec_test load
   cmd

.. _section-4:

0.5.4 (2014-07-11)
------------------

-  Bugfix: made test with filenames OS agnostic (issue #162).

.. _section-5:

0.5.3 (2012-11-03)
------------------

-  Added ability to customize string coercion (e.g. to have None render
   as ``''``) (issue #130).
-  Added Renderer.render_name() to render a template by name (issue
   #122).
-  Added TemplateSpec.template_path to specify an absolute path to a
   template (issue #41).
-  Added option of raising errors on missing tags/partials:
   ``Renderer(missing_tags='strict')`` (issue #110).
-  Added support for finding and loading templates by file name in
   addition to by template name (issue #127). [xgecko]
-  Added a ``parse()`` function that yields a printable, pre-compiled
   parse tree.
-  Added support for rendering pre-compiled templates.
-  Added Python 3.3 to the list of supported versions.
-  Added support for `PyPy <http://pypy.org/>`__ (issue #125).
-  Added support for `Travis CI <http://travis-ci.org>`__ (issue #124).
   [msabramo]
-  Bugfix: ``defaults.DELIMITERS`` can now be changed at runtime (issue
   #135). [bennoleslie]
-  Bugfix: exceptions raised from a property are no longer swallowed
   when getting a key from a context stack (issue #110).
-  Bugfix: lambda section values can now return non-ascii, non-unicode
   strings (issue #118).
-  Bugfix: allow ``test_pystache.py`` and ``tox`` to pass when run from
   a downloaded sdist (i.e. without the spec test directory).
-  Convert HISTORY and README files from reST to Markdown.
-  More robust handling of byte strings in Python 3.
-  Added Creative Commons license for David Phillips's logo.

.. _section-6:

0.5.2 (2012-05-03)
------------------

-  Added support for dot notation and version 1.1.2 of the spec (issue
   #99). [rbp]
-  Missing partials now render as empty string per latest version of
   spec (issue #115).
-  Bugfix: falsey values now coerced to strings using str().
-  Bugfix: lambda return values for sections no longer pushed onto
   context stack (issue #113).
-  Bugfix: lists of lambdas for sections were not rendered (issue #114).

.. _section-7:

0.5.1 (2012-04-24)
------------------

-  Added support for Python 3.1 and 3.2.
-  Added tox support to test multiple Python versions.
-  Added test script entry point: pystache-test.
-  Added \__version_\_ package attribute.
-  Test harness now supports both YAML and JSON forms of Mustache spec.
-  Test harness no longer requires nose.

.. _section-8:

0.5.0 (2012-04-03)
------------------

This version represents a major rewrite and refactoring of the code base
that also adds features and fixes many bugs. All functionality and
nearly all unit tests have been preserved. However, some backwards
incompatible changes to the API have been made.

Below is a selection of some of the changes (not exhaustive).

Highlights:

-  Pystache now passes all tests in version 1.0.3 of the `Mustache
   spec <https://github.com/mustache/spec>`__. [pvande]
-  Removed View class: it is no longer necessary to subclass from View
   or from any other class to create a view.
-  Replaced Template with Renderer class: template rendering behavior
   can be modified via the Renderer constructor or by setting attributes
   on a Renderer instance.
-  Added TemplateSpec class: template rendering can be specified on a
   per-view basis by subclassing from TemplateSpec.
-  Introduced separation of concerns and removed circular dependencies
   (e.g. between Template and View classes, cf. `issue
   #13 <https://github.com/defunkt/pystache/issues/13>`__).
-  Unicode now used consistently throughout the rendering process.
-  Expanded test coverage: nosetests now runs doctests and ~105 test
   cases from the Mustache spec (increasing the number of tests from 56
   to ~315).
-  Added a rudimentary benchmarking script to gauge performance while
   refactoring.
-  Extensive documentation added (e.g. docstrings).

Other changes:

-  Added a command-line interface. [vrde]
-  The main rendering class now accepts a custom partial loader (e.g. a
   dictionary) and a custom escape function.
-  Non-ascii characters in str strings are now supported while
   rendering.
-  Added string encoding, file encoding, and errors options for decoding
   to unicode.
-  Removed the output encoding option.
-  Removed the use of markupsafe.

Bug fixes:

-  Context values no longer processed as template strings.
   [jakearchibald]
-  Whitespace surrounding sections is no longer altered, per the spec.
   [heliodor]
-  Zeroes now render correctly when using PyPy. [alex]
-  Multline comments now permitted. [fczuardi]
-  Extensionless template files are now supported.
-  Passing ``**kwargs`` to ``Template()`` no longer modifies the
   context.
-  Passing ``**kwargs`` to ``Template()`` with no context no longer
   raises an exception.

.. _section-9:

0.4.1 (2012-03-25)
------------------

-  Added support for Python 2.4. [wangtz, jvantuyl]

.. _section-10:

0.4.0 (2011-01-12)
------------------

-  Add support for nested contexts (within template and view)
-  Add support for inverted lists
-  Decoupled template loading

.. _section-11:

0.3.1 (2010-05-07)
------------------

-  Fix package

.. _section-12:

0.3.0 (2010-05-03)
------------------

-  View.template_path can now hold a list of path
-  Add {{& blah}} as an alias for {{{ blah }}}
-  Higher Order Sections
-  Inverted sections

.. _section-13:

0.2.0 (2010-02-15)
------------------

-  Bugfix: Methods returning False or None are not rendered
-  Bugfix: Don't render an empty string when a tag's value is 0.
   [enaeseth]
-  Add support for using non-callables as View attributes.
   [joshthecoder]
-  Allow using View instances as attributes. [joshthecoder]
-  Support for Unicode and non-ASCII-encoded bytestring output.
   [enaeseth]
-  Template file encoding awareness. [enaeseth]

.. _section-14:

0.1.1 (2009-11-13)
------------------

-  Ensure we're dealing with strings, always
-  Tests can be run by executing the test file directly

.. _section-15:

0.1.0 (2009-11-12)
------------------

-  First release

License
=======

Copyright (C) 2012 Chris Jerdonek. All rights reserved.

Copyright (c) 2009 Chris Wanstrath

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

.. |CI| image:: https://github.com/PennyDreadfulMTG/pystache/actions/workflows/ci.yml/badge.svg
   :target: https://github.com/PennyDreadfulMTG/pystache/actions/workflows/ci.yml
.. |Conda| image:: https://github.com/PennyDreadfulMTG/pystache/actions/workflows/conda.yml/badge.svg
   :target: https://github.com/PennyDreadfulMTG/pystache/actions/workflows/conda.yml
.. |Coverage| image:: https://github.com/PennyDreadfulMTG/pystache/actions/workflows/coverage.yml/badge.svg
   :target: https://github.com/PennyDreadfulMTG/pystache/actions/workflows/coverage.yml
.. |Security check - Bandit| image:: https://github.com/PennyDreadfulMTG/pystache/actions/workflows/bandit.yml/badge.svg
   :target: https://github.com/PennyDreadfulMTG/pystache/actions/workflows/bandit.yml
.. |Release| image:: https://github.com/PennyDreadfulMTG/pystache/actions/workflows/release.yml/badge.svg
   :target: https://github.com/PennyDreadfulMTG/pystache/actions/workflows/release.yml
.. |pre-commit| image:: https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white
   :target: https://github.com/pre-commit/pre-commit
.. |Maintainability| image:: https://api.codeclimate.com/v1/badges/a8fa1bf4638bfc6581b6/maintainability
   :target: https://codeclimate.com/github/PennyDreadfulMTG/pystache/maintainability
.. |Latest release| image:: https://img.shields.io/github/v/release/PennyDreadfulMTG/pystache?include_prereleases
   :target: https://github.com/PennyDreadfulMTG/pystache/releases/latest
.. |License| image:: https://img.shields.io/github/license/PennyDreadfulMTG/pystache
   :target: https://github.com/PennyDreadfulMTG/pystache/blob/master/LICENSE
.. |Python| image:: https://img.shields.io/badge/python-3.6+-blue.svg
   :target: https://www.python.org/downloads/
.. |image1| image:: https://i.creativecommons.org/l/by-sa/3.0/88x31.png
